/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "gsm.h"

/* 
   convert_from_gsm was taken from the scmxx-0.6.3.5
   (which was made by Hendrik Sattler) source, and
   then modified to my needs!
   go get it at: http://www.hendrik-sattler.de/scmxx/
*/
QString convert_from_gsm (const QString& input)  {
    QString retval;
    for (uint i = 0;i < input.length(); i++) {
  	const ushort& a = input[i].unicode();
	switch (a) {
	    default:
		retval += a;
		break;
	    case 0x80: // commercial at
		/*
		 * we expect this as 0x80 instead of 0x00
		 * to avoid problem with C's char*
		 */
		retval +=  0x0040;
		break;
	    case 0x01: // pound sign
		retval +=  0x00a3;
		break;
	    case 0x02: // dollar sign
		retval +=  0x0024;
		break;
	    case 0x03: // yen sign
		retval +=  0x00a5;
		break;
	    case 0x04: // small 'e' with grave
		retval +=  0x00e8;
		break;
	    case 0x05: // small 'e' with acute
		retval +=  0x00e9;
		break;
	    case 0x06: // small 'u' with grave
		retval +=  0x00f9;
		break;
	    case 0x07: // small 'i' with grave
		retval +=  0x00ec;
		break;
	    case 0x08: // small 'o' with grave
		retval +=  0x00f2;
		break;
	    case 0x09: // small/capital 'c' with cedilla (we map to small)
		retval +=  0x00e7; // or 0x00c7 ?
		break;
	    case 0x0b: // capital 'O' with stroke
		retval +=  0x00d8;
		break;
	    case 0x0c: // small 'o' with stroke
		retval +=  0x00f8;
		break;
	    case 0x0e: // capital 'A' with ring above
		retval +=  0x00c5;
		break;
	    case 0x0f: // small 'a' with ring above
		retval +=  0x00e5;
		break;
	    case 0x10: // capital delta
		retval +=  0x0394;
		break;
	    case 0x11: // underscore / low line
		retval +=  0x005f;
		break;
	    case 0x12: // capital phi
		retval +=  0x03a6;
		break;
	    case 0x13: // capital gamma
		retval +=  0x0393;
		break;
	    case 0x14: // capital lambda
		retval +=  0x039b;
		break;
	    case 0x15: // capital omega
		retval +=  0x03a9;
		break;
	    case 0x16: // capital pi
		retval +=  0x03a0;
		break;
	    case 0x17: // capital psi
		retval +=  0x03a8;
		break;
	    case 0x18: // capital sigma
		retval +=  0x03a3;
		break;
	    case 0x19: // capital theta
		retval +=  0x0398;
		break;
	    case 0x1a: // capital xi
		retval +=  0x039e;
		break;
	    case 0x1b:
		++i;
		switch (a) {
		    default: // non-breaking space
			--i;
			retval +=  0x00a0;
			break;
		    case 0x1a: // form feed
			retval +=  0x000c;
			break;
		    case 0x14: // circumflex accent
			retval +=  0x005e;
			break;
		    case 0x28: // left curly bracket
			retval +=  0x007b;
			break;
		    case 0x29: // right curly bracket
			retval += 0x007d;
			break;
		    case 0x2f: // reverse solidus
			retval += 0x005c;
			break;
		    case 0x3c: // left square bracket
			retval += 0x005b;
			break;
		    case 0x3d: // tilde
			retval += 0x007e;
			break;
		    case 0x3e: // right square bracket
			retval += 0x005d;
			break;
		    case 0x40: // vertical line
			retval += 0x007c;
			break;
		    case 0x65: // Euro sign
			retval += 0x20ac;
			break;
		}
		break;
	    case 0x1c: // capital 'AE'
		retval +=  0x00c6;
		break;
	    case 0x1d: // small 'ae'
		retval +=  0x00e6;
		break;
	    case 0x1e: // small sharp s (german)
		retval +=  0x00df;
		break;
	    case 0x1f: // capital 'E' with acute
		retval +=  0x00c9;
		break;
	    case 0x24: // currency sign
		//retval +=  0x20ac;
		retval +=  0x00a4;
		break;
	    case 0x40: // inverted exclamation mark
		retval +=  0x00a1;
		break;
	    case 0x5b: // capital 'A' with diaeresis
		retval +=  0x00c4;
		break;
	    case 0x5c: // capital 'O' with diaeresis
		retval +=  0x00d6;
		break;
	    case 0x5d: // capital 'N' with tilde
		retval +=  0x00d1;
		break;
	    case 0x5e: // capital 'U' with diaeresis
		retval +=  0x00dc;
		break;
	    case 0x5f: // section sign
		retval +=  0x00a7;
		break;
	    case 0x60: // inverted question mark
		retval +=  0x00bf;
		break;
	    case 0x7b: // small 'a' with diaeresis
		retval +=  0x00e4;
		break;
	    case 0x7c: // small 'o' with diaeresis
		retval +=  0x00f6;
		break;
	    case 0x7d: // small 'n' with tilde
		retval +=  0x00f1;
		break;
	    case 0x7e: // small 'u' with diaeresis
		retval +=  0x00fc;
		break;
	    case 0x7f: // small 'a' with grave
		retval +=  0x00e0;
		break;
	}
    }
    return retval;
}

/*
   convert_to_gsm was taken from the scmxx-0.6.3.5
   (which was made by Hendrik Sattler) source, and
   then modified to my needs!
   go get it at: http://www.hendrik-sattler.de/scmxx/
*/
QString convert_to_gsm (const QString& input)  {
  QString retval;

  for (uint i = 0; i < input.length(); i++) {
  	const ushort& a = input[i].unicode();
    switch (a) {
    default:
      if (a < 0x0080 && (
				/*
				 * see Unicode table for descriptions
				 */
				a == 0x000A ||
				a == 0x000D ||
				(a >= 0x0020 && a <= 0x0023) ||
				(a >= 0x0025 && a <= 0x003f) ||
				(a >= 0x0041 && a <= 0x005a) ||
				(a >= 0x0061 && a <= 0x007a))) {
	retval += a;
      }
      break;
    case 0x0040: // commercial at
      /*
       * we map this to 0x80 instead of 0x00 to avoid problem with C's char*
       * just make sure you only use the last 7 bits of these values,
       * you must do that anyway
       */
      retval +=  0x80;
      break;
    case 0x00a3: // pound sign
      retval +=  0x01;
      break;
    case 0x0024: // dollar sign
      retval +=  0x02;
      break;
    case 0x00a5: // yen sign
      retval +=  0x03;
      break;
    case 0x00e8: // small 'e' with grave
      retval +=  0x04;
      break;
    case 0x00e9: // small 'e' with acute
      retval +=  0x05;
      break;
    case 0x00f9: // small 'u' with grave
      retval +=  0x06;
      break;
    case 0x00ec: // small 'i' with grave
      retval +=  0x07;
      break;
    case 0x00f2: // small 'o' with grave
      retval +=  0x08;
      break;
    case 0x00c7: // capital 'C' with cedilla
    case 0x00e7: // small 'c' with cedilla
      retval +=  0x09;
      break;
    case 0x00d8: // capital 'O' with stroke
      retval +=  0x0b;
      break;
    case 0x00f8: // small 'o' with stroke
      retval +=  0x0c;
      break;
    case 0x00c5: // capital 'A' with ring above
      retval +=  0x0e;
      break;
    case 0x00e5: // small 'a' with ring above
      retval +=  0x0f;
      break;
    case 0x0394: // capital delta
      retval +=  0x10;
      break;
    case 0x005f: // underscore / low line
      retval +=  0x11;
      break;
    case 0x03a6: // capital phi
      retval +=  0x12;
      break;
    case 0x0393: // capital gamma
      retval +=  0x13;
      break;
    case 0x039b: // capital lambda
      retval +=  0x14;
      break;
    case 0x03a9: // capital omega
      retval +=  0x15;
      break;
    case 0x03a0: // capital pi
      retval +=  0x16;
      break;
    case 0x03a8: // capital psi
      retval +=  0x17;
      break;
    case 0x03a3: // capital sigma
      retval +=  0x18;
      break;
    case 0x0398: // capital theta
      retval +=  0x19;
      break;
    case 0x039e: // capital xi
      retval +=  0x1a;
      break;
    case 0x000c: // form feed
      retval +=  0x1b;
      retval +=  0x0a;
      break;
    case 0x005e: // circumflex accent
      retval +=  0x1b;
      retval +=  0x14;
      break;
    case 0x007b: // left curly bracket
      retval +=  0x1b;
      retval +=  0x28;
      break;
    case 0x007d: // right curly bracket
      retval +=  0x1b;
      retval +=  0x29;
      break;
    case 0x005c: // reverse solidus (back slash)
      retval +=  0x1b;
      retval +=  0x2f;
      break;
    case 0x005b: // left square bracket
      retval +=  0x1b;
      retval +=  0x3c;
      break;
    case 0x007e: // tilde
      retval +=  0x1b;
      retval +=  0x3d;
      break;
    case 0x005d: // right square bracket
      retval +=  0x1b;
      retval +=  0x3e;
      break;
    case 0x007c: // vertical line
      retval +=  0x1b;
      retval +=  0x40;
      break;
    case 0x20ac: // Euro sign
      retval +=  0x1b;
      retval +=  0x65;
      break;
    case 0x00c6: // capital 'AE'
      retval +=  0x1c;
      break;
    case 0x00e6: // small 'ae'
      retval +=  0x1d;
      break;
    case 0x00df: // small sharp s (german)
      retval +=  0x1e;
      break;
    case 0x00c9: // capital 'E' with acute
      retval +=  0x1f;
      break;
    case 0x00a4: // currency sign
    case 0x00b0: // degree sign (not offical, preference of author)
      retval +=  0x24;
      break;
    case 0x00a1: // inverted exclamation mark
      retval +=  0x40;
      break;
    case 0x0391: // capital alpha
      retval +=  0x41;
      break;
    case 0x0392: // capital beta
      retval +=  0x42;
      break;
    case 0x0395: // capital epsilon
      retval +=  0x45;
      break;
    case 0x0397: // capital eta
      retval +=  0x48;
      break;
    case 0x0399: // capital iota
      retval +=  0x49;
      break;
    case 0x039a: // capital kappa
      retval +=  0x4b;
      break;
    case 0x039c: // capital mu
      retval +=  0x4d;
      break;
    case 0x039d: // capital nu
      retval +=  0x4e;
      break;
    case 0x039f: // capital omicron
      retval +=  0x4f;
      break;
    case 0x03a1: // capital rho
      retval +=  0x50;
      break;
    case 0x03a4: // capital tau
      retval +=  0x54;
      break;
    case 0x03a5: // capital upsilon
      retval +=  0x55;
      break;
    case 0x03a7: // capital chi
      retval +=  0x58;
      break;
    case 0x0396: // capital zeta
      retval +=  0x5a;
      break;
    case 0x00c4: // capital 'A' with diaeresis
      retval +=  0x5b;
      break;
    case 0x00d6: // capital 'O' with diaeresis
      retval +=  0x5c;
      break;
    case 0x00d1: // capital 'N' with tilde
      retval +=  0x5d;
      break;
    case 0x00dc: // capital 'U' with diaeresis
      retval +=  0x5e;
      break;
    case 0x00a7: // section sign
      retval +=  0x5f;
      break;
    case 0x00bf: // inverted question mark
      retval +=  0x60;
      break;
    case 0x00e4: // small 'a' with diaeresis
      retval +=  0x7b;
      break;
    case 0x00f6: // small 'o' with diaeresis
      retval +=  0x7c;
      break;
    case 0x00f1: // small 'n' with tilde
      retval +=  0x7d;
      break;
    case 0x00fc: // small 'u' with diaeresis
      retval +=  0x7e;
      break;
    case 0x00e0: // small 'a' with grave
      retval +=  0x7f;
      break;
    }
  }
  return retval;
}
