/***************************************************************************
                          settingsdlg.cpp  -  description
                             -------------------
    begin                : Wed Mar 5 2003
    copyright            : (C) 2003 by Tobias Bayer
    email                : t.bayer@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "settingsdlg.h"
#include <klocale.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <kpushbutton.h>
#include <kconfig.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qvalidator.h>

SettingsDlg::SettingsDlg(QWidget *parent, const char *name) : QWidget(0L, name)
{
  this->setCaption(i18n("Leif Settings"));

  v = new QIntValidator(0,3600,this);
  grid = new QGridLayout(this, 1, 1, 11, 6);

  label1 = new QLabel(this);
  label1->setText(i18n("Device:"));
  grid->addWidget(label1, 0, 0);
  
  deviceEdit = new QLineEdit(this);
  grid->addWidget(deviceEdit, 0 , 1);

  connectAtStart = new QCheckBox(this);
  connectAtStart->setText(i18n("Connect at start"));
  grid->addWidget(connectAtStart, 1, 0);

  label4 = new QLabel(this);
  label4->setText(i18n("Country Code:"));
  grid->addWidget(label4, 2, 0);

  countryCodeEdit = new QLineEdit(this);
  grid->addWidget(countryCodeEdit, 2 , 1);

  label5 = new QLabel(this);
  label5->setText(i18n("Refresh Interval [s]:"));
  grid->addWidget(label5, 3, 0);

  refreshEdit = new QLineEdit(this);
  refreshEdit->setValidator(v);
  grid->addWidget(refreshEdit, 3 , 1);

  btOpts = new QGroupBox(this);
  btOpts->setTitle(i18n("Additional Bluetooth options"));
  btOpts->setColumns(2);
  grid->addMultiCellWidget(btOpts, 4,4,0,2);

  label2 = new QLabel(btOpts);
  label2->setText(i18n("Address:"));

  btAddress = new QLineEdit(btOpts);

  label3 = new QLabel(btOpts);
  label3->setText(i18n("OBEX channel:"));

  btChannel = new QLineEdit(btOpts);
  
  okButton = new KPushButton(this);
  okButton->setText(i18n("OK"));
  grid->addWidget(okButton, 5,0);
  
  cancelButton = new KPushButton(this);
  cancelButton->setText(i18n("Cancel"));
  grid->addWidget(cancelButton, 5,1);
  
  connect(okButton, SIGNAL(clicked()), this, SLOT(saveSettings()));
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelSettings())); 

  KConfig *settings = kapp->config();
  settings->setGroup("Main");
  deviceEdit->setText(settings->readEntry("Device"));
  connectAtStart->setChecked(settings->readNumEntry("ConnectStart"));
	countryCodeEdit->setText(settings->readEntry("CountryCode"));
	refreshEdit->setText(settings->readEntry("UpdateInterval"));
  settings->setGroup("Bluetooth");
  btAddress->setText(settings->readEntry("Address"));
  btChannel->setText(settings->readEntry("OBEX_Channel"));
	
#ifndef HAVE_LIBBLUETOOTH
  btOpts->setEnabled(FALSE);
#endif
}

void SettingsDlg::saveSettings()
{
  KConfig *settings = kapp->config();
  settings->setGroup("Main");
  settings->writeEntry("Device", deviceEdit->text());

  if (connectAtStart->isChecked())
    settings->writeEntry("ConnectStart", "1");
  else
    settings->writeEntry("ConnectStart", "0");

	settings->writeEntry("CountryCode", countryCodeEdit->text());
	settings->writeEntry("UpdateInterval", refreshEdit->text());
  settings->setGroup("Bluetooth");
  settings->writeEntry("Address", btAddress->text());
  settings->writeEntry("OBEX_Channel", btChannel->text());

  settings->sync();

  emit settingsUpdated();
  this->close();
}

void SettingsDlg::cancelSettings()
{
  this->close();
}


SettingsDlg::~SettingsDlg()
{
}
