/***************************************************************************
                          sms.cpp  -  description
                             -------------------
    begin                : Don Mr 20 2003
    copyright            : (C) 2003 by Tobias Bayer
    email                : t.bayer@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sms.h"
#include <stdio.h>

SMS::SMS()
{
  
}

QString SMS::decodeBCD(QString toSwap)
{
  QString swapped="";
  int i;

  for (i=0;i<=toSwap.length();i+=2)
  {
    swapped.append(toSwap.mid(i+1,1));
    swapped.append(toSwap.mid(i,1));
  }

  
  return swapped;   
}


int SMS::parse()
{
  int pos;
  
  smscLen = PDU.left(2).toInt(0, 16);
  smscType = PDU.mid(2,2);
  smsc = PDU.mid(4, (smscLen*2) -2);
  smsc = decodeBCD(smsc);
  if (smsc.contains("F"))
    smsc.truncate(smsc.length()-1);

  smsType=PDU.mid((smscLen*2)+2,2).toInt(0,16);
  pos=(smscLen*2)+4;
  if (smsType&1)	// sent SMS?
  	pos += 2;				// skip 1 additional byte

  senderLen = (PDU.mid(pos,2)).toInt(0,16);
  senderType = (PDU.mid(pos+2,2));
  if (senderLen % 2)
    senderLen++;
  sender = PDU.mid(pos+4, senderLen);
  sender = decodeBCD(sender);
  if (sender.contains("F"))
    sender.truncate(sender.length()-1);
	if (senderType=="91")
		sender.prepend("+");
  pos +=4+senderLen;

  //skip 2 bits (protocol identifier) and 2 bits (data coding scheme)
  pos +=4;

  if (!(smsType&1))
  {
  	timeStamp = PDU.mid(pos, 14);  
  	timeStamp = decodeBCD(timeStamp);
  	//timeStamp is now in the following format: YYMMDDHHMMSSZZ with ZZ is GMT +ZZ/4  
  	pos +=14;
  }
  else if (!(smsType&64))
  	pos += 2;
   
  messageLen = (PDU.mid(pos, 2)).toInt(0,16);
  pos +=2;
	if (smsType&64) // LONG SMS
	{
		udhLength=PDU.mid(pos, 2).toInt(0,16);
		pos +=2;
		iei=PDU.mid(pos, 2).toInt(0,16);
		pos +=2;
		ie_length=PDU.mid(pos, 2).toInt(0,16);
		pos +=2;
		for (int i=0;i<ie_length;i++)
		{
				ie[i]=PDU.mid(pos, 2).toInt(0,16);
				pos +=2;
		}
		pos-=2*(udhLength+1);
	}
  message = PDU.mid(pos);
  message = decodeMessage(message);
	if (smsType&64)
	{
		int toskip=(udhLength+1)*8/7;
		if ((udhLength+1)*8%7)
			toskip++;
		message=message.mid(toskip);
  } 
  return 0;  
}

QString SMS::decodeMessage(QString encodedMessage)
{
  QString decodedMessage="";
  unsigned char data[281];
  unsigned char dec[161];
  int i;
  int c=0;
  int offset;
  int first;
  int end;
  
  for (i=0;i<encodedMessage.length();i+=2)
  {
    data[c] = (encodedMessage.mid(i, 2)).toInt(0,16);
    c++;
  }

  offset = -1;
  c = 0;

  for(i=0;i<messageLen;i++)
  {
    if (i%8)
    {
      first = data[i-1-offset];
      first >>= 8 - (i % 8);
      end = data[i - offset];
      end <<= (i % 8);
      *(dec + c) = (first | end) & 127;      
    }
    else
    {
      ++offset;
      *(dec + c) = data[i - offset] & 127;
    }

    if (*(dec + c) == 0)
      *(dec + c) = 128;     

    c++;
  }
  
  dec[c] = '\0';

  //Ok, 7bit message decoded, now convert to human readable signs

  for(i=0;i<strlen((char*)dec);i++)
  {
    switch(dec[i])
    {

      case 1:
        decodedMessage.append("");
        break;

      case 2:
        decodedMessage.append("$");
        break;

      case 3:
        decodedMessage.append("");
        break;

      case 4:
        decodedMessage.append("");
        break;

      case 5:
        decodedMessage.append("");
        break;

      case 6:
        decodedMessage.append("");
        break;

      case 7:
        decodedMessage.append("");
        break;

      case 8:
        decodedMessage.append("");
        break;

      case 9:
        decodedMessage.append("");
        break;

      case 10:
      //line feed
        break;

      case 11:
        decodedMessage.append("");
        break;

      case 12:
        decodedMessage.append("");
        break;

      case 13:
        //carriage return
        break;

      case 14:
        decodedMessage.append("");
        break;

      case 15:
        decodedMessage.append("");
        break;

      case 16:
        //captial DELTA
        break;

      case 17:
        decodedMessage.append("_");
        break;

      case 18:
        //captial PHI
        break;

      case 19:
        //captial GAMMA
        break;

      case 20:
        //captial LAMBDA
        break;

      case 21:
        //captial OMEGA
        break;

      case 22:
        //captial PI
        break;

      case 23:
        //captial PSI
        break;

      case 24:
        //captial SIGMA
        break;

      case 25:
        //captial THETA
        break;

      case 26:
        //captial XI
        break;

      case 27:  //escape to extension table
        i++;
        switch (dec[i])
        {
          case 10:
            //form feed
            break;

          case 20:
            decodedMessage.append("^");
            break;

          case 40:
            decodedMessage.append("{");
            break;

          case 41:
            decodedMessage.append("}");
            break;

          case 47:
            decodedMessage.append("\\");
            break;

          case 60:
            decodedMessage.append("[");
            break;

          case 61:
            decodedMessage.append("~");
            break;

          case 62:
            decodedMessage.append("]");
            break;

          case 64:
            decodedMessage.append("|");
            break;

          case 101:
            //EURO SIGN decodedMessage.append("");
            break;
        }        
        break;

      case 28:
        decodedMessage.append("");
        break;

      case 29:
        decodedMessage.append("");
        break;

      case 30:
        decodedMessage.append("");
        break;

      case 31:
        decodedMessage.append("");
        break;

      case 32:
        decodedMessage.append(" ");
        break;

      case 33:
        decodedMessage.append("!");
        break;

      case 34:
        decodedMessage.append("\"");
        break;

      case 35:
        decodedMessage.append("#");
        break;

      case 36:
        //currency sign decodedMessage.append("");
        break;

      case 37:
        decodedMessage.append("%");
        break;

      case 38:
        decodedMessage.append("&");
        break;

      case 39:
        decodedMessage.append("'");
        break;

      case 40:
        decodedMessage.append("(");
        break;

      case 41:
        decodedMessage.append(")");
        break;

      case 42:
        decodedMessage.append("*");
        break;

      case 43:
        decodedMessage.append("+");
        break;

      case 44:
        decodedMessage.append(",");
        break;

      case 45:
        decodedMessage.append("-");
        break;

      case 46:
        decodedMessage.append(".");
        break;

      case 47:
        decodedMessage.append("/");
        break;

      case 48:
        decodedMessage.append("0");
        break;

      case 49:
        decodedMessage.append("1");
        break;

      case 50:
        decodedMessage.append("2");
        break;

      case 51:
        decodedMessage.append("3");
        break;

      case 52:
        decodedMessage.append("4");
        break;

      case 53:
        decodedMessage.append("5");
        break;

      case 54:
        decodedMessage.append("6");
        break;

      case 55:
        decodedMessage.append("7");
        break;

      case 56:
        decodedMessage.append("8");
        break;

      case 57:
        decodedMessage.append("9");
        break;

      case 58:
        decodedMessage.append(":");
        break;

      case 59:
        decodedMessage.append(";");
        break;

      case 60:
        decodedMessage.append("<");
        break;

      case 61:
        decodedMessage.append("=");
        break;

      case 62:
        decodedMessage.append(">");
        break;

      case 63:
        decodedMessage.append("?");
        break;

      case 64:
        decodedMessage.append("");
        break;

      case 65:
        decodedMessage.append("A");
        break;

      case 66:
        decodedMessage.append("B");
        break;

      case 67:
        decodedMessage.append("C");
        break;

      case 68:
        decodedMessage.append("D");
        break;

      case 69:
        decodedMessage.append("E");
        break;

      case 70:
        decodedMessage.append("F");
        break;                                

      case 71:
        decodedMessage.append("G");
        break;

      case 72:
        decodedMessage.append("H");
        break;

      case 73:
        decodedMessage.append("I");
        break;

      case 74:
        decodedMessage.append("J");
        break;

      case 75:
        decodedMessage.append("K");
        break;

      case 76:
        decodedMessage.append("L");
        break;

      case 77:
        decodedMessage.append("M");
        break;

      case 78:
        decodedMessage.append("N");
        break;

      case 79:
        decodedMessage.append("O");
        break;

      case 80:
        decodedMessage.append("P");
        break;

      case 81:
        decodedMessage.append("Q");
        break;

      case 82:
        decodedMessage.append("R");
        break;

      case 83:
        decodedMessage.append("S");
        break;

      case 84:
        decodedMessage.append("T");
        break;

      case 85:
        decodedMessage.append("U");
        break;

      case 86:
        decodedMessage.append("V");
        break;

      case 87:
        decodedMessage.append("W");
        break;

      case 88:
        decodedMessage.append("X");
        break;

      case 89:
        decodedMessage.append("Y");
        break;

      case 90:
        decodedMessage.append("Z");
        break;

      case 91:
        decodedMessage.append("");
        break;

      case 92:
        decodedMessage.append("");
        break;

      case 93:
        decodedMessage.append("");
        break;

      case 94:
        decodedMessage.append("");
        break;

      case 95:
        decodedMessage.append("");
        break;

      case 96:
        decodedMessage.append("");
        break;

      case 97:
        decodedMessage.append("a");
        break;

      case 98:
        decodedMessage.append("b");
        break;

      case 99:
        decodedMessage.append("c");
        break;

      case 100:
        decodedMessage.append("d");
        break;

      case 101:
        decodedMessage.append("e");
        break;

      case 102:
        decodedMessage.append("f");
        break;

      case 103:
        decodedMessage.append("g");
        break;

      case 104:
        decodedMessage.append("h");
        break;

      case 105:
        decodedMessage.append("i");
        break;

      case 106:
        decodedMessage.append("j");
        break;

      case 107:
        decodedMessage.append("k");
        break;

      case 108:
        decodedMessage.append("l");
        break;

      case 109:
        decodedMessage.append("m");
        break;

      case 110:
        decodedMessage.append("n");
        break;

      case 111:
        decodedMessage.append("o");
        break;

      case 112:
        decodedMessage.append("p");
        break;

      case 113:
        decodedMessage.append("q");
        break;

      case 114:
        decodedMessage.append("r");
        break;

      case 115:
        decodedMessage.append("s");
        break;

      case 116:
        decodedMessage.append("t");
        break;

      case 117:
        decodedMessage.append("u");
        break;

      case 118:
        decodedMessage.append("v");
        break;

      case 119:
        decodedMessage.append("w");
        break;

      case 120:
        decodedMessage.append("x");
        break;

      case 121:
        decodedMessage.append("y");
        break;

      case 122:
        decodedMessage.append("z");
        break;

      case 123:
        decodedMessage.append("");
        break;

      case 124:
        decodedMessage.append("");
        break;

      case 125:
        decodedMessage.append("");
        break;

      case 126:
        decodedMessage.append("");
        break;

      case 127:
        decodedMessage.append("");
        break;

      case 128:
        decodedMessage.append("@");
        break;

      default:
        break;
    } 
  }  
  return decodedMessage;
}

QString SMS::encodeMessage(QString decodedMessage, int longMessage)
{
  QString encodedMessage ="";
  QString num;
  unsigned char enc[281];
  int i, first, end;

  messageLen = decodedMessage.length();
  
  for (i=0;i<decodedMessage.length();i++)
  {
    switch(decodedMessage.at(i).latin1())
    {

      case '':
        enc[i] = 0x01;
        break;

      case '$':
        enc[i] = 0x02;
        break;

      case '':
        enc[i] = 3;
        break;

      case '':
        enc[i] = 4;
        break;

      case '':
        enc[i] = 5;
        break;

      case '':
        enc[i] = 6;
        break;

      case '':
        enc[i] = 7;
        break;

      case '':
        enc[i] = 8;
        break;

      case '':
        enc[i] = 9;
        break;

      /*case line feed:
        enc[i] = 10;
        break;*/

      case '':
        enc[i] = 11;
        break;

      case '':
        enc[i] = 12;
        break;

      /*case carriage return:
        enc[i] = 13;
        break;*/

      case '':
        enc[i] = 14;
        break;

      case '':
        enc[i] = 15;
        break;

      /*case capital DELTA:
        enc[i] = 16;
        break;                    */

      case '_':
        enc[i] = 17;
        break;

        
  /*  GREEK ALPHABET
      case
        enc[i] = 18;
        break;

      case '':
        enc[i] = 19;
        break;

      case '':
        enc[i] = 21;
        break;

      case '':
        enc[i] = 22;
        break;

      case '':
        enc[i] = 23;
        break;

      case '':
        enc[i] = 24;
        break;

      case '':
        enc[i] = 25;
        break;

      case '':
        enc[i] = 26;
        break;
  */
      case '^':
        enc[i] = 27;
        i++;
        enc[i] = 20;
        break;

      case '{':
        enc[i] = 27;
        i++;
        enc[i] = 40;
        break;

      case '}':
        enc[i] = 27;
        i++;
        enc[i] = 41;
        break;

      case '\\':
        enc[i] = 27;
        i++;
        enc[i] = 47;
        break;

      case '[':
        enc[i] = 27;
        i++;
        enc[i] = 60;
        break;

      case '~':
        enc[i] = 27;
        i++;
        enc[i] = 61;
        break;

      case ']':
        enc[i] = 27;
        i++;
        enc[i] = 62;
        break;

      case '|':
        enc[i] = 27;
        i++;
        enc[i] = 64;
        break;

/*EURO SIGN      case '':
        enc[i] = 27;
        i++;
        enc[i] = 40;
        break;   */

      case '':
        enc[i] = 28;
        break;

      case '':
        enc[i] = 29;
        break;

      case '':
        enc[i] = 30;
        break;

      case '':
        enc[i] = 31;
        break;

      case ' ':
        enc[i] = 32;
        break;

      case '!':
        enc[i] = 33;
        break;

      case '"':
        enc[i] = 34;
        break;

      case '#':
        enc[i] = 35;
        break;

      //CURRENCY SIGN


      case '%':
        enc[i] = 37;
        break;

      case '&':
        enc[i] = 38;
        break;

      case '\'':
        enc[i] = 39;
        break;

      case '(':
        enc[i] = 40;
        break;

      case ')':
        enc[i] = 41;
        break;

      case '*':
        enc[i] = 42;
        break;

      case '+':
        enc[i] = 43;
        break;

      case ',':
        enc[i] = 44;
        break;

      case '-':
        enc[i] = 45;
        break;

      case '.':
        enc[i] = 46;
        break;

      case '/':
        enc[i] = 47;
        break;

      case '0':
        enc[i] = 48;
        break;

      case '1':
        enc[i] = 49;
        break;

      case '2':
        enc[i] = 50;
        break;

      case '3':
        enc[i] = 51;
        break;

      case '4':
        enc[i] = 52;
        break;

      case '5':
        enc[i] = 53;
        break;

      case '6':
        enc[i] = 54;
        break;

      case '7':
        enc[i] = 55;
        break;

      case '8':
        enc[i] = 56;
        break;

      case '9':
        enc[i] = 57;
        break;

      case ':':
        enc[i] = 58;
        break;

      case ';':
        enc[i] = 59;
        break;

      case '<':
        enc[i] = 60;
        break;

      case '=':
        enc[i] = 61;
        break;

      case '>':
        enc[i] = 62;
        break;

      case '?':
        enc[i] = 63;
        break;

      case '':
        enc[i] = 64;
        break;

      case 'A':
        enc[i] = 65;
        break;

      case 'B':
        enc[i] = 66;
        break;

      case 'C':
        enc[i] = 67;
        break;

      case 'D':
        enc[i] = 68;
        break;

      case 'E':
        enc[i] = 69;
        break;

      case 'F':
        enc[i] = 70;
        break;

      case 'G':
        enc[i] = 71;
        break;

      case 'H':
        enc[i] = 72;
        break;

      case 'I':
        enc[i] = 73;
        break;

      case 'J':
        enc[i] = 74;
        break;

      case 'K':
        enc[i] = 75;
        break;

      case 'L':
        enc[i] = 76;
        break;

      case 'M':
        enc[i] = 77;
        break;

      case 'N':
        enc[i] = 78;
        break;

      case 'O':
        enc[i] = 79;
        break;

      case 'P':
        enc[i] = 80;
        break;

      case 'Q':
        enc[i] = 81;
        break;

      case 'R':
        enc[i] = 82;
        break;

      case 'S':
        enc[i] = 83;
        break;

      case 'T':
        enc[i] = 84;
        break;

      case 'U':
        enc[i] = 85;
        break;

      case 'V':
        enc[i] = 86;
        break;

      case 'W':
        enc[i] = 87;
        break;

      case 'X':
        enc[i] = 88;
        break;

      case 'Y':
        enc[i] = 89;
        break;

      case 'Z':
        enc[i] = 90;
        break;

      case '':
        enc[i] = 91;
        break;

      case '':
        enc[i] = 92;
        break;

      case '':
        enc[i] = 93;
        break;

      case '':
        enc[i] = 94;
        break;

      case '':
        enc[i] = 95;
        break;

      case '':
        enc[i] = 96;
        break;

      case 'a':
        enc[i] = 97;
        break;

      case 'b':
        enc[i] = 98;
        break;

      case 'c':
        enc[i] = 99;
        break;

      case 'd':
        enc[i] = 100;
        break;

      case 'e':
        enc[i] = 101;
        break;

      case 'f':
        enc[i] = 102;
        break;

      case 'g':
        enc[i] = 103;
        break;

      case 'h':
        enc[i] = 104;
        break;

      case 'i':
        enc[i] = 105;
        break;

      case 'j':
        enc[i] = 106;
        break;

      case 'k':
        enc[i] = 107;
        break;

      case 'l':
        enc[i] = 108;
        break;

      case 'm':
        enc[i] = 109;
        break;

      case 'n':
        enc[i] = 110;
        break;

      case 'o':
        enc[i] = 111;
        break;

      case 'p':
        enc[i] = 112;
        break;

      case 'q':
        enc[i] = 113;
        break;

      case 'r':
        enc[i] = 114;
        break;

      case 's':
        enc[i] = 115;
        break;

      case 't':
        enc[i] = 116;
        break;

      case 'u':
        enc[i] = 117;
        break;

      case 'v':
        enc[i] = 118;
        break;

      case 'w':
        enc[i] = 119;
        break;

      case 'x':
        enc[i] = 120;
        break;

      case 'y':
        enc[i] = 121;
        break;

      case 'z':
        enc[i] = 122;
        break;

      case '':
        enc[i] = 123;
        break;

      case '':
        enc[i] = 124;
        break;

      case '':
        enc[i] = 125;
        break;

      case '':
        enc[i] = 126;
        break;

      case '':
        enc[i] = 127;
        break;      
        
      default:
        break;
    }
  }

  enc[i] = '\0';
  
  
  //Now, we have to encode the message to its 7bit representation
  for (i=0; i<strlen((char*)enc); i++)
		if (i==0 && longMessage) // special treatnmen - would "normally" be skipped since %8==7
  	{
   		first=(enc[0]&127)<<1;
   		num.setNum(first, 16);
	    if (first < 16)
  	    encodedMessage.append("0");
    	encodedMessage.append(num);     
  	}
  	else
  	{
	    end = enc[i] & 127;
  	  end >>= ((i-longMessage) % 8);
    	first = enc[i+1] & 127;
	    first <<= 7 - ((i-longMessage) % 8);
  	  first &= 255;
    	num.setNum((first|end), 16);
	    if ((first|end) < 16)
  	    encodedMessage.append("0");
    	encodedMessage.append(num);
	    if (((i-longMessage) % 8) == 6)
  	    i++;
  }
   
  return encodedMessage;
}


void SMS::createSendPDU(int index, int id)
{
	int longMessage=message.length()>160;
  QString p = longMessage?"004100":"000100";
  QString len;
  QString udLen;
  QString encodedMessage;
	QString recipientType = "85";
	QString myRecipient=recipient;
	
  //Set address length
  if (myRecipient.left(1)=="+")
  {
    myRecipient=myRecipient.mid(1);
    recipientType = "91";
  }
  recipientLen = myRecipient.length();
  len.setNum(recipientLen,16);
  if (recipientLen < 16)
    p.append("0");
  p.append(len);

  //Set number format 91 or 81
  p.append(recipientType);

  //Add recipient BCD encoded number
  if ((recipientLen%2) != 0)
    myRecipient.append("F");

  p.append(decodeBCD(myRecipient));

  //Protocol id and data coding scheme
  p.append("0000");

  //User data length (before this, the message must be encoded)
  //encode Message
  if (longMessage)
  	encodedMessage = encodeMessage(message.mid(153*(index-1),153*index),longMessage);
  else
  	encodedMessage = encodeMessage(message,longMessage);  

  //Length
  if (longMessage)
  {
  	udLen.setNum(messageLen+7, 16);
  	if (messageLen < 16)
    	udLen.prepend("0");
  	p.append(udLen);
    p.append("050003");
    int chars=message.length();
    int numSMS=chars/153;
    if (chars%153)
    	numSMS++;
    udLen.setNum(id, 16);
    if (id<16)
    	udLen.prepend("0");
    p.append(udLen);
    udLen.setNum(numSMS, 16);
    if (numSMS<16)
    	udLen.prepend("0");
    p.append(udLen);
    udLen.setNum(index, 16);
    if (index<16)
    	udLen.prepend("0");
    p.append(udLen);
  }
  else
  {
  	udLen.setNum(messageLen, 16);
  	if (messageLen < 16)
    	udLen.prepend("0");
  	p.append(udLen);
  }

  //Message
  p.append(encodedMessage);

  //Set the new PDU
  PDU = p;
}

SMS::~SMS()
{
  
}
